package me.km.blocks;

import java.util.Random;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.FallingBlock;
import net.minecraft.block.SlabBlock;
import net.minecraft.block.material.Material;
import net.minecraft.entity.item.FallingBlockEntity;
import net.minecraft.particles.BlockParticleData;
import net.minecraft.particles.ParticleTypes;
import net.minecraft.state.properties.SlabType;
import net.minecraft.util.Direction;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IWorld;
import net.minecraft.world.IWorldReader;
import net.minecraft.world.World;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import net.minecraftforge.common.ToolType;

public class BlockGravelSlab extends SlabBlock
{
    public BlockGravelSlab(Properties properties)
    {
        super(properties);
    }

    @Override
    public ToolType getHarvestTool(BlockState state)
    {
        return ToolType.SHOVEL;
    }

    @Override
    public void onBlockAdded(BlockState p_220082_1_, World w, BlockPos pos, BlockState p_220082_4_, boolean p_220082_5_)
    {
        w.getPendingBlockTicks().scheduleTick(pos, this, this.tickRate(w));
    }

    @Override
    public BlockState updatePostPlacement(BlockState state, Direction facing, BlockState facingState, IWorld w, BlockPos currentPos, BlockPos facingPos)
    {
        w.getPendingBlockTicks().scheduleTick(currentPos, this, this.tickRate(w));
        return super.updatePostPlacement(state, facing, facingState, w, currentPos, facingPos);
    }

    @Override
    public void tick(BlockState state, World w, BlockPos pos, Random random)
    {
        if(!w.isRemote)
        {
            this.checkFallable(w, pos);
        }
    }

    private void checkFallable(World w, BlockPos pos)
    {
        BlockState state = w.getBlockState(pos);
        if((w.isAirBlock(pos.down()) || FallingBlock.canFallThrough(w.getBlockState(pos.down()))) && pos.getY() >= 0)
        {
            if(!w.isRemote)
            {
                FallingBlockEntity fbe = new FallingBlockEntity(w, pos.getX() + 0.5, pos.getY(), pos.getZ() + 0.5, state);
                w.addEntity(fbe);
            }

        }
        else if(state == getDefaultState().with(TYPE, SlabType.TOP) && pos.getY() >= 0)
        {
            if(!w.isRemote)
            {
                FallingBlockEntity fbe = new FallingBlockEntity(w, 
                        pos.getX() + 0.5, pos.getY(), pos.getZ() + 0.5, 
                        w.getBlockState(pos).with(TYPE, SlabType.BOTTOM));
                w.addEntity(fbe);
            }

        }
    }
    
    @Override
    public int tickRate(IWorldReader w)
    {
        return 2;
    }

    @OnlyIn(Dist.CLIENT)
    @Override
    public void animateTick(BlockState state, World w, BlockPos pos, Random rand)
    {
        if(rand.nextInt(16) == 0)
        {
            BlockPos blockpos = pos.down();
            if(w.isAirBlock(blockpos) || FallingBlock.canFallThrough(w.getBlockState(blockpos)))
            {
                double x = pos.getX() + rand.nextFloat();
                double y = pos.getY() - 0.05;
                double z = pos.getZ() + rand.nextFloat();
                w.addParticle(new BlockParticleData(ParticleTypes.FALLING_DUST, state), x, y, z, 0.0, 0.0, 0.0);
            }
        }

    }

    @OnlyIn(Dist.CLIENT)
    public int getDustColor(BlockState state)
    {
        return -8356741;
    }
}
