package me.km.entities;

import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.function.Consumer;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.DownloadImageBuffer;
import net.minecraft.client.renderer.texture.DynamicTexture;
import net.minecraft.client.renderer.texture.NativeImage;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import org.apache.logging.log4j.LogManager;

@OnlyIn(Dist.CLIENT)
public class HumanSkinLoader
{
    public final static HumanSkinLoader INSTANCE = new HumanSkinLoader();

    private static final ResourceLocation TEXTURE_STEVE = new ResourceLocation("textures/entity/steve.png");

    private final HashMap<String, ResourceLocation> skins = new HashMap<>();
    private final HashSet<String> loading = new HashSet<>();

    private final TextureManager manager = Minecraft.getInstance().getRenderManager().textureManager;
    private final DownloadImageBuffer converter = new DownloadImageBuffer();

    private HumanSkinLoader()
    {
    }

    public ResourceLocation getTexture(String name)
    {
        ResourceLocation loc = skins.get(name);
        if(loc != null)
        {
            return loc;
        }
        if(loading.add(name))
        {
            downloadSkin(name, image ->
            {
                ResourceLocation rloc = manager.getDynamicTextureLocation("skin_" + name, new DynamicTexture(image));
                skins.put(name, rloc);
                loading.remove(name);
            });
        }
        return TEXTURE_STEVE;
    }

    private void downloadSkin(String name, Consumer<NativeImage> delayed)
    {
        new Thread(() ->
        {
            HttpURLConnection httpurlconnection = null;
            try
            {
                //httpurlconnection = (HttpURLConnection)(new URL("http://skins.minecraft.net/MinecraftSkins/" + name + ".png"))
                httpurlconnection = (HttpURLConnection) (new URL("http://skins.hammerle.me/" + name + ".png"))
                        .openConnection(Minecraft.getInstance().getProxy());
                httpurlconnection.setDoInput(true);
                httpurlconnection.setDoOutput(false);
                httpurlconnection.connect();

                int code = httpurlconnection.getResponseCode();
                if(code != 200)
                {
                    // Failed own server, trying crafatar
                    httpurlconnection.disconnect();
                    httpurlconnection = (HttpURLConnection) (new URL("https://crafatar.com/skins/" + name + ".png"))
                            .openConnection(Minecraft.getInstance().getProxy());
                    httpurlconnection.setDoInput(true);
                    httpurlconnection.setDoOutput(false);
                    httpurlconnection.connect();
                    code = httpurlconnection.getResponseCode();
                    if(code != 200)
                    {
                        LogManager.getLogger().warn("Server response code did return " + code + ", skin servers might be down.");
                        return;
                    }
                }

                NativeImage image = converter.parseUserSkin(NativeImage.read(httpurlconnection.getInputStream()));
                Minecraft.getInstance().enqueue(() -> delayed.accept(image));
            }
            catch(Exception ex)
            {
                LogManager.getLogger().warn("Error occurred when downloading skin, however, skin servers seem to be up.");
            }
            finally
            {
                if(httpurlconnection != null)
                {
                    httpurlconnection.disconnect();
                }
            }
        }).start();
    }
}
