package me.km.entities;

import com.mojang.blaze3d.platform.GlStateManager;
import net.minecraft.client.renderer.entity.EntityRendererManager;
import net.minecraft.client.renderer.entity.LivingRenderer;
import net.minecraft.client.renderer.entity.layers.ArrowLayer;
import net.minecraft.client.renderer.entity.layers.BipedArmorLayer;
import net.minecraft.client.renderer.entity.layers.ElytraLayer;
import net.minecraft.client.renderer.entity.layers.HeadLayer;
import net.minecraft.client.renderer.entity.layers.HeldItemLayer;
import net.minecraft.client.renderer.entity.model.BipedModel;
import net.minecraft.item.CrossbowItem;
import net.minecraft.item.ItemStack;
import net.minecraft.item.Items;
import net.minecraft.item.UseAction;
import net.minecraft.util.Hand;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;

@OnlyIn(Dist.CLIENT)
public class RenderHuman extends LivingRenderer<EntityHuman, ModelHuman>
{
    private final ModelHuman steve;
    private final ModelHuman alex;

    public RenderHuman(EntityRendererManager erm)
    {
        super(erm, new ModelHuman(0.0f, false), 0.5f);
        this.addLayer(new BipedArmorLayer<>(this, new BipedModel(0.5F), new BipedModel(1.0F)));
        this.addLayer(new HeldItemLayer<>(this));
        this.addLayer(new ArrowLayer<>(this));
        this.addLayer(new HeadLayer<>(this));
        this.addLayer(new ElytraLayer<>(this));

        this.steve = this.getEntityModel();
        this.alex = new ModelHuman(0, true);
    }

    @Override
    public void doRender(EntityHuman ent, double x, double y, double z, float entityYaw, float partialTicks)
    {
        this.setModelVisibilities(ent);
        GlStateManager.setProfile(GlStateManager.Profile.PLAYER_SKIN);
        if(ent.isSlim())
        {
            this.entityModel = alex;
            super.doRender(ent, x, y, z, entityYaw, partialTicks);
            this.entityModel = steve;
        }
        else
        {
            super.doRender(ent, x, y, z, entityYaw, partialTicks);
        }
        GlStateManager.unsetProfile(GlStateManager.Profile.PLAYER_SKIN);
    }

    private void setModelVisibilities(EntityHuman ent)
    {
        ModelHuman playermodel = this.getEntityModel();
        
        playermodel.setVisible(true);
        playermodel.bipedHeadwear.showModel = true;
        playermodel.bipedBodyWear.showModel = true;
        playermodel.bipedLeftLegwear.showModel = true;
        playermodel.bipedRightLegwear.showModel = true;
        playermodel.bipedLeftArmwear.showModel = true;
        playermodel.bipedRightArmwear.showModel = true;
        playermodel.isSneak = false;
        ItemStack mainHand = ent.getHeldItemMainhand();
        ItemStack offHand = ent.getHeldItemOffhand();
        playermodel.rightArmPose = getArmPose(ent, mainHand, offHand, Hand.MAIN_HAND);
        playermodel.leftArmPose = getArmPose(ent, mainHand, offHand, Hand.OFF_HAND);
    }

    private BipedModel.ArmPose getArmPose(EntityHuman ent, ItemStack mainStack, ItemStack offStack, Hand hand)
    {
        BipedModel.ArmPose pose = BipedModel.ArmPose.EMPTY;
        ItemStack itemstack = hand == Hand.MAIN_HAND ? mainStack : offStack;
        if(!itemstack.isEmpty())
        {
            pose = BipedModel.ArmPose.ITEM;
            if(ent.getItemInUseCount() > 0)
            {
                UseAction useaction = itemstack.getUseAction();
                if(useaction == UseAction.BLOCK)
                {
                    pose = BipedModel.ArmPose.BLOCK;
                }
                else if(useaction == UseAction.BOW)
                {
                    pose = BipedModel.ArmPose.BOW_AND_ARROW;
                }
                else if(useaction == UseAction.SPEAR)
                {
                    pose = BipedModel.ArmPose.THROW_SPEAR;
                }
                else if(useaction == UseAction.CROSSBOW && hand == ent.getActiveHand())
                {
                    pose = BipedModel.ArmPose.CROSSBOW_CHARGE;
                }
            }
            else
            {
                if(mainStack.getItem() == Items.CROSSBOW && CrossbowItem.isCharged(mainStack))
                {
                    pose = BipedModel.ArmPose.CROSSBOW_HOLD;
                }

                if(offStack.getItem() == Items.CROSSBOW && CrossbowItem.isCharged(offStack) && mainStack.getItem().getUseAction(mainStack) == UseAction.NONE)
                {
                    pose = BipedModel.ArmPose.CROSSBOW_HOLD;
                }
            }
        }

        return pose;
    }

    @Override
    public ResourceLocation getEntityTexture(EntityHuman ent)
    {
        return ent.getTexture();
    }

    @Override
    protected void preRenderCallback(EntityHuman ent, float partialTickTime)
    {
        GlStateManager.scalef(0.9375F, 0.9375F, 0.9375F);
    }
}
