package me.km.events;

import java.util.function.BiPredicate;
import me.hammerle.snuviscript.code.Script;
import me.km.Server;
import me.km.utils.Location;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.world.IWorld;

public class PlayerMoveData 
{
    private static int idCounter = 0;
    
    private final Script sc;
    private final BiPredicate<ServerPlayerEntity, Object> c;
    private final int coolDown;
    private int ticks;
    private int livingTime;
    
    private Object data;
    
    private final IWorld w;
    private final double minX;
    private final double minY;
    private final double minZ;
    private final double maxX;
    private final double maxY;
    private final double maxZ;
    
    private final int id;
    
    private PlayerMoveData(Script sc, BiPredicate<ServerPlayerEntity, Object> c, Location l1, Location l2, int cooldown, int livingTime, Object data)
    {
        this.id = idCounter++;
        this.sc = sc;
        this.c = c;
        this.coolDown = cooldown;
        this.ticks = cooldown;
        this.livingTime = livingTime;
        this.w = l1.getWorld();
        this.minX = Math.min(l1.getX(), l2.getX());
        this.minY = Math.min(l1.getY(), l2.getY());
        this.minZ = Math.min(l1.getZ(), l2.getZ());
        this.maxX = Math.max(l1.getX(), l2.getX());
        this.maxY = Math.max(l1.getY(), l2.getY());
        this.maxZ = Math.max(l1.getZ(), l2.getZ());
        this.data = data;
        //System.out.println("X DATA FROM " + minX +" TO " + maxX);
        //System.out.println("Y DATA FROM " + minY +" TO " + maxY);
        //System.out.println("Z DATA FROM " + minZ +" TO " + maxZ);
    }
    
    public PlayerMoveData(Script sc, Location l1, Location l2, int cooldown, int livingTime)
    {
        this(sc, null, l1, l2, cooldown, livingTime, null);
    }
    
    public PlayerMoveData(BiPredicate<ServerPlayerEntity, Object> c, Location l1, Location l2, int cooldown, int livingTime, Object data)
    {
        this(null, c, l1, l2, cooldown, livingTime, data);
    }

    public int getId() 
    {
        return id;
    }
    
    public boolean isSameScript(Script sc)
    {
        return this.sc == sc;
    }
    
    public Object getData()
    {
        return data;
    }
    
    public boolean tickLiving()
    {
        if(livingTime == -1)
        {
            return false;
        }
        livingTime--;
        return livingTime < 0;
    }
    
    public boolean tick()    
    {
        ticks--;
        if(ticks > 0)
        {
            return true;
        }
        ticks = coolDown;
        return false;
    }
    
    public boolean check(ServerPlayerEntity p)    
    {
        if(p.world != w || p.posX < minX || p.posX > maxX || p.posZ < minZ || p.posZ > maxZ || p.posY < minY || p.posY > maxY)
        {
            return false;
        }
        if(sc != null)
        {
            Server.scriptEvents.onPlayerMove(p, id);
        }
        if(c != null)
        {
            if(c.test(p, data))
            {
                return true;
            }
        }
        return coolDown == -1;
    }
}
