package me.km.inventory;

import me.km.networking.ModPacketHandler;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.inventory.container.Container;
import net.minecraft.inventory.container.IContainerListener;
import net.minecraft.inventory.container.Slot;
import net.minecraft.inventory.container.CraftingResultSlot;
import net.minecraft.item.ItemStack;
import net.minecraft.network.play.server.SSetSlotPacket;
import net.minecraft.network.play.server.SWindowItemsPacket;
import net.minecraft.util.NonNullList;
import net.minecraft.util.text.ITextComponent;

public abstract class CustomContainerBase extends Container
{
    protected final PlayerEntity viewer;
    protected final boolean isServerSide;
    protected final InventoryBase inv;
    private final int numRows;
    private final ITextComponent title;

    private static int getId(ServerPlayerEntity p)
    {
        p.getNextWindowId();
        return p.currentWindowId;
    }
    
    protected CustomContainerBase(InventoryBase inv, PlayerEntity viewer, ITextComponent title, int id)
    {
        super(null, id);
        // basic stuff
        this.title = title;
        this.viewer = viewer;
        this.isServerSide = viewer instanceof ServerPlayerEntity;
        this.inv = inv;
        this.numRows = inv.getRows();
        int i = (this.numRows - 4) * 18;
        
        // inventory slots
        int counter = 0;
        for(int y = 0; y < this.numRows; y++)
        {
            for(int x = 0; x < 9; x++)
            {
                if(inv.isSlotValid(x, y))
                {
                    super.addSlot(new Slot(inv, counter, 8 + x * 18, 18 + y * 18));
                    counter++;
                }
            }
        }

        PlayerInventory pInv = this.viewer.inventory;
        // plaver inventory slots
        for(int y = 0; y < 3; y++)
        {
            for(int x = 0; x < 9; x++)
            {
                super.addSlot(new Slot(pInv, x + y * 9 + 9, 8 + x * 18, 103 + y * 18 + i));
            }
        }
        for(int x = 0; x < 9; x++)
        {
            super.addSlot(new Slot(pInv, x, 8 + x * 18, 161 + i));
        }
    }
    
    protected CustomContainerBase(InventoryBase inv, ServerPlayerEntity viewer, ITextComponent title)
    {
        this(inv, viewer, title, getId(viewer));
    }

    public ITextComponent getTitle()
    {
        return title;
    }

    public InventoryBase getInventoryBase() 
    {
        return inv;
    }
    
    public void closeSafe()
    {
        viewer.closeScreen();
    }
    
    public void openForPlayer()
    {
        if(isServerSide)
        {
            ServerPlayerEntity p = (ServerPlayerEntity) viewer;
            if(p.openContainer != p.container)
            {
                p.closeScreen();
            }
            p.openContainer = this;
            ModPacketHandler.sendCustomInventory(p, windowId, title, inv);
            p.openContainer.addListener(new IContainerListener() 
            {
                @Override
                public void sendAllContents(Container containerToSend, NonNullList<ItemStack> itemsList) 
                {
                    p.connection.sendPacket(new SWindowItemsPacket(containerToSend.windowId, itemsList));
                    p.connection.sendPacket(new SSetSlotPacket(-1, -1, p.inventory.getItemStack()));
                }

                @Override
                public void sendSlotContents(Container containerToSend, int slotInd, ItemStack stack) 
                {
                    if(!(containerToSend.getSlot(slotInd) instanceof CraftingResultSlot))
                    {
                        if(containerToSend == p.container)
                        {
                            CriteriaTriggers.INVENTORY_CHANGED.trigger(p, p.inventory);
                        }
                        if(!p.isChangingQuantityOnly)
                        {
                            p.connection.sendPacket(new SSetSlotPacket(containerToSend.windowId, slotInd, stack));
                        }
                    }
                }

                @Override
                public void sendWindowProperty(Container containerIn, int varToUpdate, int newValue) 
                {
                }
            });
        }
    }

    @Override
    public boolean canInteractWith(PlayerEntity p)
    {
        return this.inv.isUsableByPlayer(p);
    }

    // ignore special slots stuff
    @Override
    public ItemStack transferStackInSlot(PlayerEntity p, int index)
    {
        ItemStack stack = ItemStack.EMPTY;
        Slot slot = this.inventorySlots.get(index);
        int size = inv.getSizeInventory();
        
        if(slot != null && slot.getHasStack())
        {
            ItemStack slotStack = slot.getStack();
            stack = slotStack.copy();
            if(index < size)
            {
                if(!this.mergeItemStack(slotStack, size, this.inventorySlots.size(), true))
                {
                    return ItemStack.EMPTY;
                }
            }
            else if(!this.mergeItemStack(slotStack, 0, size, false))
            {
                return ItemStack.EMPTY;
            }
            if(slotStack.isEmpty())
            {
                slot.putStack(ItemStack.EMPTY);
            }
            else
            {
                slot.onSlotChanged();
            }
        }
        return stack;
    }
    
    @Override
    public boolean canMergeSlot(ItemStack stack, Slot slot)
    {
        return inv.getSlotStatus(slot.slotNumber) == 1;
    }
    
    @Override
    public boolean canDragIntoSlot(Slot slot)
    {
        return inv.getSlotStatus(slot.slotNumber) == 1;
    }
    
    @Override
    protected boolean mergeItemStack(ItemStack stack, int startIndex, int endIndex, boolean reverseDirection)
    {
        boolean flag = false;
        int i = startIndex;

        if (reverseDirection)
        {
            i = endIndex - 1;
        }

        if (stack.isStackable())
        {
            while (!stack.isEmpty())
            {
                if (reverseDirection)
                {
                    if (i < startIndex)
                    {
                        break;
                    }
                }
                else if (i >= endIndex)
                {
                    break;
                }
                
                if(inv.getSlotStatus(i) == 1)
                {
                    Slot slot = this.inventorySlots.get(i);
                    ItemStack itemstack = slot.getStack();

                    if (!itemstack.isEmpty() && ItemStack.areItemStackTagsEqual(stack, itemstack))
                    {
                        int j = itemstack.getCount() + stack.getCount();
                        int maxSize = Math.min(slot.getSlotStackLimit(), stack.getMaxStackSize());

                        if (j <= maxSize)
                        {
                            stack.setCount(0);
                            itemstack.setCount(j);
                            slot.onSlotChanged();
                            flag = true;
                        }
                        else if (itemstack.getCount() < maxSize)
                        {
                            stack.shrink(maxSize - itemstack.getCount());
                            itemstack.setCount(maxSize);
                            slot.onSlotChanged();
                            flag = true;
                        }
                    }
                }

                if (reverseDirection)
                {
                    --i;
                }
                else
                {
                    ++i;
                }
            }
        }

        if (!stack.isEmpty())
        {
            if (reverseDirection)
            {
                i = endIndex - 1;
            }
            else
            {
                i = startIndex;
            }

            while (true)
            {
                if (reverseDirection)
                {
                    if (i < startIndex)
                    {
                        break;
                    }
                }
                else if (i >= endIndex)
                {
                    break;
                }

                if(inv.getSlotStatus(i) == 1)
                {
                    Slot slot1 = this.inventorySlots.get(i);
                    ItemStack itemstack1 = slot1.getStack();

                    if (itemstack1.isEmpty() && slot1.isItemValid(stack))
                    {
                        if (stack.getCount() > slot1.getSlotStackLimit())
                        {
                            slot1.putStack(stack.split(slot1.getSlotStackLimit()));
                        }
                        else
                        {
                            slot1.putStack(stack.split(stack.getCount()));
                        }

                        slot1.onSlotChanged();
                        flag = true;
                        break;
                    }
                }

                if (reverseDirection)
                {
                    --i;
                }
                else
                {
                    ++i;
                }
            }
        }

        return flag;
    }
}
