package me.km.networking;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.function.Supplier;
import me.km.world.WorldManager;
import net.minecraft.network.PacketBuffer;
import net.minecraftforge.fml.network.NetworkEvent;

public class DimensionSync
{
    private static class Data
    {
        private String registry;
        private int id;
    }
    
    private final ArrayList<Data> data = new ArrayList<>();
    
    private DimensionSync()
    {
    }
    
    public static DimensionSync create()
    {
        DimensionSync sync = new DimensionSync();
        WorldManager.forEachType(type -> 
        {
            if(type.isVanilla())
            {
                return;
            }
            Data d = new Data();
            d.id = type.getId() + 1;
            d.registry = type.getRegistryName().toString();
            sync.data.add(d);
        });
        return sync;
    }
    
    public static void writeBytes(DimensionSync ds, PacketBuffer buffer)
    {
        buffer.writeInt(ds.data.size());
        for(Data d : ds.data)
        {
            byte[] registry = d.registry.getBytes(StandardCharsets.UTF_8);
            buffer.writeInt(registry.length);
            buffer.writeBytes(registry);
            buffer.writeInt(d.id);
        }
    }
    
    public static DimensionSync fromBytes(PacketBuffer buf)
    {
        DimensionSync ds = new DimensionSync();
        int amount = buf.readInt();

        for(int i = 0; i < amount; i++)
        {
            Data d = new Data();
            int length = buf.readInt();
            d.registry = buf.readBytes(length).toString(StandardCharsets.UTF_8);
            d.id = buf.readInt();
            ds.data.add(d);
        }
        return ds;
    }
    
    public static void handle(DimensionSync ds, Supplier<NetworkEvent.Context> context)
    {
        context.get().enqueueWork(() -> 
        {
            WorldManager.reset();
            for(Data d : ds.data)
            {
                WorldManager.registerUnsafe(d.id, d.registry);
            }
        });
        context.get().setPacketHandled(true);
    }
}
