package me.km.snuviscript;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.UUID;
import me.hammerle.snuviscript.code.SnuviUtils;
import me.km.KajetansMod;
import me.km.databank.DataBank;
import me.km.playerbank.PlayerManager;

public class ScriptBank implements IScriptBank
{   
    private final DataBank databank;
    private final PreparedStatement getVar;
    private final PreparedStatement getMapValue;
    private final PreparedStatement getDualMapValue;
    
    public ScriptBank(DataBank databank, PlayerManager manager) 
    {
        this.databank = databank;
        databank.execute("CREATE TABLE IF NOT EXISTS scriptdata ("
                + "id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, "
                + "player_id INT NOT NULL, "
                + "var VARCHAR(20) NOT NULL, "
                + "value VARCHAR(256) NOT NULL, "
                + "UNIQUE INDEX (player_id,var), "
                + "FOREIGN KEY (player_id) REFERENCES minecraft.players(id) ON DELETE RESTRICT);");
        
        databank.execute("CREATE TABLE IF NOT EXISTS scriptmaps ("
                + "id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, "
                + "map VARCHAR(255) NOT NULL, "
                + "keyname VARCHAR(255) NOT NULL, "
                + "value VARCHAR(255) NOT NULL, "
                + "INDEX (map), "
                + "UNIQUE INDEX (map,keyname));");
        
        databank.execute("CREATE TABLE IF NOT EXISTS scriptdualmaps ("
                + "id INT NOT NULL AUTO_INCREMENT PRIMARY KEY, "
                + "map VARCHAR(255) NOT NULL, "
                + "keyname VARCHAR(255) NOT NULL, "
                + "seckeyname VARCHAR(255) NOT NULL, "
                + "value VARCHAR(255) NOT NULL, "
                + "INDEX (map), "
                + "UNIQUE INDEX (map,keyname,seckeyname));");
        
        getVar = databank.prepareStatement("SELECT value from scriptdata WHERE player_id = ? and var = ?;");
        getMapValue = databank.prepareStatement("SELECT value from scriptmaps WHERE map = ? AND keyname = ?;");
        getDualMapValue = databank.prepareStatement("SELECT value from scriptdualmaps WHERE map = ? AND keyname = ? AND seckeyname = ?;");
    }

    @Override
    public void setVar(String value, String var, int playerId)
    {
        try(PreparedStatement setVar = databank.prepareStatement(
                "INSERT INTO scriptdata (player_id,var,value) SELECT ?, ?, ? ON DUPLICATE KEY UPDATE scriptdata.value=?;"))
        {
            setVar.setInt(1, playerId);
            setVar.setString(2, var);
            setVar.setString(3, value);
            setVar.setString(4, value);
            setVar.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }

    @Override
    public void deleteVar(String var, int playerId)
    {
        try(PreparedStatement deleteVar = databank.prepareStatement(
                "DELETE FROM scriptdata WHERE player_id = ? AND var = ?;"))
        {
            deleteVar.setInt(1, playerId);
            deleteVar.setString(2, var);
            deleteVar.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public Object getVar(String var, int playerId, Object error)
    {
        try
        {
            getVar.setInt(1, playerId);
            getVar.setString(2, var);
            try(ResultSet rs = getVar.executeQuery())
            {
                if(rs.next())
                {
                    return SnuviUtils.convert(rs.getString(1));
                }
                return error;
            }
            catch(SQLException ex)
            {
                ex.printStackTrace();
            }
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
        return error;
    } 
    
    @Override
    public Object getVar(String var, int playerId)
    {
        return getVar(var, playerId, null);
    }    
    
    @Override
    public void addMapElement(String map, String key, String value)
    {
        try(PreparedStatement addMapElement = databank.prepareStatement(
                "INSERT INTO scriptmaps (map,keyname,value) SELECT ?, ?, ? ON DUPLICATE KEY UPDATE value=?;"))
        {
            addMapElement.setString(1, map);
            addMapElement.setString(2, key);
            addMapElement.setString(3, value);
            addMapElement.setString(4, value);
            addMapElement.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void removeMapElement(String map, String key)
    {
        try(PreparedStatement removeMapElement = databank.prepareStatement(
                "DELETE FROM scriptmaps WHERE map = ? AND keyname = ?;"))
        {
            removeMapElement.setString(1, map);
            removeMapElement.setString(2, key);
            removeMapElement.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void removeMap(String map)
    {
        try(PreparedStatement removeMap = databank.prepareStatement(
                "DELETE FROM scriptmaps WHERE map = ?;"))
        {
            removeMap.setString(1, map);
            removeMap.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public Object getMapValue(String map, String key)
    {
        try
        {
            getMapValue.setString(1, map);
            getMapValue.setString(2, key);
            try(ResultSet rs = getMapValue.executeQuery())
            {
                if(rs.next())
                {
                    return SnuviUtils.convert(rs.getString(1));
                }
                return null;
            }
            catch(SQLException ex)
            {
                ex.printStackTrace();
            }
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
        return null;
    } 
    
    @Override
    public void addDualMapElement(String map, String key, String key2, String value)
    {
        try(PreparedStatement addDualMapElement = databank.prepareStatement(
                "INSERT INTO scriptdualmaps (map,keyname,seckeyname,value) SELECT ?, ?, ?, ? ON DUPLICATE KEY UPDATE value=?;"))
        {
            addDualMapElement.setString(1, map);
            addDualMapElement.setString(2, key);
            addDualMapElement.setString(3, key2);
            addDualMapElement.setString(4, value);
            addDualMapElement.setString(5, value);
            addDualMapElement.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void removeDualMapElement(String map, String key)
    {
        try(PreparedStatement removeDualMapElement = databank.prepareStatement(
                "DELETE FROM scriptdualmaps WHERE map = ? AND keyname = ?;"))
        {
            removeDualMapElement.setString(1, map);
            removeDualMapElement.setString(2, key);
            removeDualMapElement.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void removeDualMap(String map)
    {
        try(PreparedStatement removeDualMap = databank.prepareStatement(
                "DELETE FROM scriptdualmaps WHERE map = ?;"))
        {
            removeDualMap.setString(1, map);
            removeDualMap.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void removeDualMapElement(String map, String key, String key2)
    {
        try(PreparedStatement removeDualMapElement2 = databank.prepareStatement(
                "DELETE FROM scriptdualmaps WHERE map = ? AND keyname = ? AND seckeyname = ?;"))
        {
            removeDualMapElement2.setString(1, map);
            removeDualMapElement2.setString(2, key);
            removeDualMapElement2.setString(3, key2);
            removeDualMapElement2.executeUpdate();
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
    } 

    @Override
    public Object getDualMapValue(String map, String key, String key2)
    {
        try
        {
            getDualMapValue.setString(1, map);
            getDualMapValue.setString(2, key);
            getDualMapValue.setString(3, key2);
            try(ResultSet rs = getDualMapValue.executeQuery())
            {
                if(rs.next())
                {
                    return SnuviUtils.convert(rs.getString(1));
                }
                return null;
            }
            catch(SQLException ex)
            {
                ex.printStackTrace();
            }
        }
        catch(SQLException ex)
        {
            ex.printStackTrace();
        }
        return null;
    } 
}
