package me.km.world;

import me.km.blocks.ModBlocks;
import net.minecraft.block.Blocks;
import net.minecraft.world.biome.Biome;
import net.minecraft.world.gen.GenerationStage;
import net.minecraft.world.gen.feature.ConfiguredFeature;
import net.minecraft.world.gen.feature.DecoratedFeatureConfig;
import net.minecraft.world.gen.feature.Feature;
import net.minecraft.world.gen.feature.OreFeatureConfig;
import net.minecraft.world.gen.placement.CountRangeConfig;
import net.minecraft.world.gen.placement.Placement;
import net.minecraftforge.registries.ForgeRegistries;

public class ModWorldGeneration
{
    public static void register()
    {
        // search for all biomes which generate ore
        for(Biome biome : ForgeRegistries.BIOMES.getValues())
        {
            for(ConfiguredFeature f : biome.getFeatures(GenerationStage.Decoration.UNDERGROUND_ORES))
            {
                if(!(f.config instanceof DecoratedFeatureConfig))
                {
                    continue;
                }
                DecoratedFeatureConfig conf = (DecoratedFeatureConfig) f.config;
                if(!(conf.feature.config instanceof OreFeatureConfig))
                {
                    continue;
                }
                OreFeatureConfig ore = (OreFeatureConfig) conf.feature.config;
                if(ore.state == Blocks.IRON_ORE.getDefaultState())
                {
                    addFeatures(biome);
                    break;
                }
            }
        }
    }
    
    private static void addFeatures(Biome biome)
    {
        //          | max   | bottom | top    | max  | spawn |
        // name     | size  | offset | offset | high | tries |
        // ---------------------------------------------------
        // coal     |    17 |      0 |      0 |  128 |    20 |
        // iron     |     9 |      0 |      0 |   64 |    20 |
        // gold     |     9 |      0 |      0 |   32 |     2 |
        // redstone |     8 |      0 |      0 |   16 |     8 |
        // diamond  |     8 |      0 |      0 |   16 |     1 |
        // lapis    |     7 |      - |      - |    - |     - |
        // copper   |     9 |      0 |      0 |  128 |    25 |
        // tin      |     6 |      0 |      0 |   96 |    12 |
        // silver   |     9 |      0 |      0 |   64 |     3 |
        
        biome.addFeature(GenerationStage.Decoration.UNDERGROUND_ORES, Biome.createDecoratedFeature(
                        Feature.ORE, new OreFeatureConfig(OreFeatureConfig.FillerBlockType.NATURAL_STONE, 
                                ModBlocks.copperOre.getDefaultState(), 9), Placement.COUNT_RANGE, 
                        new CountRangeConfig(25, 0, 0, 128)));
        biome.addFeature(GenerationStage.Decoration.UNDERGROUND_ORES, Biome.createDecoratedFeature(
                        Feature.ORE, new OreFeatureConfig(OreFeatureConfig.FillerBlockType.NATURAL_STONE, 
                                ModBlocks.tinOre.getDefaultState(), 6), Placement.COUNT_RANGE, 
                        new CountRangeConfig(12, 0, 0, 96)));
        biome.addFeature(GenerationStage.Decoration.UNDERGROUND_ORES, Biome.createDecoratedFeature(
                        Feature.ORE, new OreFeatureConfig(OreFeatureConfig.FillerBlockType.NATURAL_STONE, 
                                ModBlocks.silverOre.getDefaultState(), 9), Placement.COUNT_RANGE, 
                        new CountRangeConfig(3, 0, 0, 64)));
    }
}